//
//  PowerGuardController.m
//  PowerGuard
//
//  Created by Phidgets on 2015-07-15.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "PowerGuardController.h"

@implementation PowerGuardController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close the example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetPowerGuard_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetPowerGuard_create(&ch);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}


#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    double minOverVoltage,maxOverVoltage;
    int powerEnabled;
    Phidget_FanMode fanMode;
    
    
    //Get information from channel which will allow us to configure the GUI properly
    PhidgetPowerGuard_getMinOverVoltage(ch, &minOverVoltage);
    PhidgetPowerGuard_getMaxOverVoltage(ch, &maxOverVoltage);
    PhidgetPowerGuard_getPowerEnabled(ch, &powerEnabled);
    PhidgetPowerGuard_getFanMode(ch, &fanMode);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [dataBox setHidden:NO];
    [settingsBox setHidden:NO];
    if(fanMode == FAN_MODE_ON)
        [fanModeCombo selectItemAtIndex:0];
    if(fanMode == FAN_MODE_OFF)
        [fanModeCombo selectItemAtIndex:1];
    if(fanMode == FAN_MODE_AUTO)
        [fanModeCombo selectItemAtIndex:2];
    //Configure the over voltage slider with data read from channel
    [overVoltageSlider setMinValue:minOverVoltage];
    [overVoltageSlider setMaxValue:maxOverVoltage];

    [minOverVoltageLabel setStringValue:[NSString stringWithFormat:@"%.2f V", minOverVoltage]];
    [maxOverVoltageLabel setStringValue:[NSString stringWithFormat:@"%.2f V", maxOverVoltage]];
    [powerEnabledCheck setState:powerEnabled];
    
    

    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Taking items out of view
    [settingsBox setHidden:YES];
    [dataBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Reset GUI
    [overVoltageText setStringValue:@"V"];
    [overVoltageSlider setDoubleValue:[overVoltageSlider minValue]];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}


static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)outputLastError {
	const char *errorString;
	char *errorDetailString = NULL;
    size_t errorDetailStringLen = 0;
    PhidgetReturnCode lastErr;
	
    if (!Phidget_getLastError(&lastErr, &errorString, NULL, &errorDetailStringLen)) {
		errorDetailString = malloc(errorDetailStringLen);
		Phidget_getLastError(&lastErr, &errorString, errorDetailString, &errorDetailStringLen);
    	[self outputError:errorDetailString];
    	free(errorDetailString);
    }
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

#pragma mark GUI controls
- (IBAction)setOverVoltage:(id)sender {
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if([currentEvent type] == NSEventTypeLeftMouseUp){
        result = PhidgetPowerGuard_setOverVoltage(ch, [overVoltageSlider doubleValue]);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
    }
    [overVoltageText setStringValue:[NSString stringWithFormat:@"%.2f V", [overVoltageSlider doubleValue]]];
}
- (IBAction)setPowerEnabled:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetPowerGuard_setPowerEnabled(ch, (int)[powerEnabledCheck state]);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
}
- (IBAction)setFanMode:(id)sender {
    PhidgetReturnCode result;
    Phidget_FanMode fanMode = FAN_MODE_AUTO;
    NSComboBox *combo = (NSComboBox *)sender;
    switch([combo indexOfSelectedItem]){
        case 0:
            fanMode = FAN_MODE_ON;
            break;
        case 1:
            fanMode = FAN_MODE_OFF;
            break;
        case 2:
            fanMode = FAN_MODE_AUTO;
            break;
        default:
            break;
    }
    result = PhidgetPowerGuard_setFanMode(ch, fanMode);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
}

@end
